@@ -19,8 +19,10 @@ def parse_output(output):
 
 def bake_ansible_playbook():
     args = ['-i', 'inventory']
+    args = ['-i', 'development-inventory']
     if settings.CONFIG_DIR != '.':
         args.extend(['-i', f'{settings.CONFIG_DIR}/inventory'])
+        args.extend(['-i', f'{settings.CONFIG_DIR}/development-inventory'])
     return sh.ansible_playbook.bake(
         *args,
         _tee=True,
@@ -130,6 +132,7 @@ class Playbook(object):
             args.append(self.vault_password_option())
         if self.share_dir != self.config_dir:
             args.extend(['-i', f'{self.config_dir}/inventory'])
+            args.extend(['-i', f'{self.config_dir}/development-inventory'])
         logger = logging.getLogger(__name__)
         return sh.ansible_playbook.bake(
             *args,
