@@ -1,7 +1,4 @@
 import base64
-from bs4 import BeautifulSoup
-import copy
-import hashlib
 import json
 import logging
 import os
@@ -14,12 +11,16 @@ import yaml
 
 from enough import settings
 from enough.common.retry import retry
+from enough.common.dotenough import Hosts
 
 log = logging.getLogger(__name__)
 
 
 class OpenStackBase(object):
 
+    INTERNAL_NETWORK = 'internal'
+    INTERNAL_NETWORK_CIDR = '10.20.30.0/24'
+
     def __init__(self, config_dir, config_file):
         self.config_dir = config_dir
         self.config_file = config_file
@@ -36,6 +37,7 @@ class Stack(OpenStackBase):
 
     def __init__(self, config_dir, config_file, definition=None):
         super().__init__(config_dir, config_file)
+        log.info(f'OS_CLIENT_CONFIG_FILE={self.config_file}')
         self.definition = definition
 
     def get_template(self):
@@ -91,10 +93,28 @@ class Stack(OpenStackBase):
             self.create()
         info = self.update()
         self.wait_for_ssh(info['ipv4'], info['port'])
+        Hosts(self.config_dir).create_or_update(self.definition['name'], info['ipv4'])
         return info
 
+    def create_internal_network(self):
+        network = self.definition.get('internal_network', OpenStackBase.INTERNAL_NETWORK)
+        cidr = self.definition.get('internal_network_cidr', OpenStackBase.INTERNAL_NETWORK_CIDR)
+        o = OpenStack(self.config_dir, self.config_file)
+        o.network_and_subnet_create(network, cidr)
+
+    def connect_internal_network(self):
+        server = self.definition['name']
+        network = self.definition.get('internal_network', OpenStackBase.INTERNAL_NETWORK)
+        o = OpenStack(self.config_dir, self.config_file)
+        if o.server_connected_to_network(server, network):
+            return
+        self.o.server.add.network(server, network)
+
     def update(self):
-        return self._create_or_update('update', self.definition.get('network', 'Ext-Net'))
+        self.create_internal_network()
+        r = self._create_or_update('update', self.definition.get('network', 'Ext-Net'))
+        self.connect_internal_network()
+        return r
 
     def create(self):
         try:
@@ -105,13 +125,11 @@ class Stack(OpenStackBase):
             # retry once because there is no way to increase the timeout and create fails often
             info = self._create_or_update('update', 'Ext-Net')
         if self.definition.get('network', 'Ext-Net') != 'Ext-Net':
-            log.info(f'create on network {self.definition["network"]}')
+            log.info(f'remove network Ext-Net')
             # wait for the interface to settle before removing it
             self.wait_for_ssh(info['ipv4'], info['port'])
             server = self.definition['name']
-            network = self.definition['network']
             self.o.server.remove.network(server, 'Ext-Net')
-            self.o.server.add.network(server, network)
 
     def delete(self):
         name = self.definition['name']
@@ -125,12 +143,19 @@ class Stack(OpenStackBase):
             assert name not in self.list(), f'{name} deletion in progress'
         wait_is_deleted()
 
+        Hosts(self.config_dir).delete(self.definition['name'])
+
+        network = self.definition.get('internal_network', OpenStackBase.INTERNAL_NETWORK)
+        o = OpenStack(self.config_dir, self.config_file)
+        o.network_delete_if_not_used(network)
+
 
 class Heat(OpenStackBase):
 
     def get_stack_definitions(self, share_dir=settings.SHARE_DIR):
         args = ['-i', f'{share_dir}/inventory',
                 '-i', f'{self.config_dir}/inventory',
+                '-i', f'{self.config_dir}/development-inventory',
                 '--vars', '--list']
         password_file = f'{self.config_dir}.pass'
         if os.path.exists(password_file):
@@ -146,6 +171,10 @@ class Heat(OpenStackBase):
             'port': h.get('ansible_port', '22'),
             'flavor': h.get('openstack_flavor', 's1-2'),
             'network': h.get('openstack_network', 'Ext-Net'),
+            'internal_network': h.get('openstack_internal_network',
+                                      OpenStackBase.INTERNAL_NETWORK),
+            'internal_network_cidr': h.get('openstack_internal_network_cidr',
+                                           OpenStackBase.INTERNAL_NETWORK_CIDR),
         }
         if 'openstack_volumes' in h:
             definition['volumes'] = h['openstack_volumes']
@@ -160,38 +189,17 @@ class Heat(OpenStackBase):
                 return False
         return True
 
+    def create_missings(self, names, public_key):
+        return self.create_or_update(Hosts(self.config_dir).missings(names), public_key)
+
     def create_or_update(self, names, public_key):
-        h = Heat(self.config_dir, self.config_file)
         r = {}
         for name in names:
-            s = Stack(self.config_dir, self.config_file, h.get_stack_definition(name))
+            s = Stack(self.config_dir, self.config_file, self.get_stack_definition(name))
             s.set_public_key(public_key)
             r[name] = s.create_or_update()
         return r
 
-    def to_inventory(self, stacks):
-        hosts = {}
-        for name, info in stacks.items():
-            hosts[name] = {
-                'ansible_host': info['ipv4'],
-            }
-        return yaml.dump(
-            {
-                'all': {
-                    'hosts': hosts,
-                },
-            }
-        )
-
-    def write_inventory(self):
-        names = Stack(self.config_dir, self.config_file).list()
-        inventory = self.to_inventory(self.create_or_update(
-            names, f'{self.config_dir}/infrastructure_key.pub'))
-        d = f'{self.config_dir}/inventory'
-        if not os.path.exists(d):
-            os.makedirs(d)
-        open(f'{d}/hosts.yml', 'w').write(inventory)
-
     def create_test_subdomain(self, domain):
         # exclusively when running from molecule
         assert os.path.exists('molecule.yml')
@@ -237,39 +245,6 @@ class OpenStack(OpenStackBase):
         super().__init__(config_dir, config_file)
         self.config = yaml.load(open(config_file))
         self.auth = self.config['clouds']['ovh']['auth']
-        self.horizon_session = None
-
-    def login_horizon(self):
-        url = 'https://horizon.cloud.ovh.net'
-
-        if self.horizon_session is None:
-            s = requests.Session()
-
-            r = s.get(url)
-            r.raise_for_status()
-            soup = BeautifulSoup(r.text, 'html.parser')
-            #  print(soup.prettify())
-            data = {}
-            for input in soup.select('form[action="/auth/login/"] input'):
-                if input.get('name') is None or input.get('value') is None:
-                    continue
-                data[input['name']] = input['value']
-            data['username'] = data['fake_email'] = self.auth['username']
-            data['password'] = data['fake_password'] = self.auth['password']
-
-            r = s.post(url + '/auth/login/', data=data, allow_redirects=False)
-            r.raise_for_status()
-            assert r.status_code == 302
-            assert r.headers['Location'] == url + '/'
-
-            r = s.get(r.headers['Location'], allow_redirects=False)
-            r.raise_for_status()
-            assert r.status_code == 302
-            assert r.headers['Location'] == url + '/project/'
-
-            self.horizon_session = s
-
-        return url, self.horizon_session
 
     @retry(OpenStackLeftovers, tries=7)
     def destroy_everything(self, prefix):
@@ -287,60 +262,44 @@ class OpenStack(OpenStackBase):
                 leftovers.append(f'image({image})')
                 self.o.image.delete(image)
 
+        for network in self.o.network.list('--format=value', '-c', 'Name', _iter=True):
+            network = network.strip()
+            if prefix is None or prefix in network:
+                leftovers.append(f'network({network})')
+                self.o.network.delete(network)
+
         if leftovers:
             raise OpenStackLeftovers('scheduled removal of ' + ' '.join(leftovers))
 
-    def region_list(self):
-        (url, s) = self.login_horizon()
-        r = s.get(f'{url}/project/')
-        r.raise_for_status()
-        soup = BeautifulSoup(r.text, 'html.parser')
-        regions = sorted([str(r.string) for r in soup.select('.region-name')])
-        return [region for region in regions if region[-1:].isdigit()]
+    def network_exists(self, name):
+        network = self.o.network.list('--format=value', '-c', 'Name', '--name', name)
+        return network.strip() == name
 
-    def _generate_clouds(self):
-        clouds = {}
-        for region in self.region_list():
-            config = copy.deepcopy(self.config)
-            config['clouds']['ovh']['region_name'] = region
-            value = yaml.dump(config)
-            key = hashlib.md5(value.encode('utf-8')).hexdigest()
-            clouds[key] = value
-        return clouds
+    def network_create(self, name):
+        if not self.network_exists(name):
+            self.o.network.create(name)
 
-    def generate_clouds(self, directory):
-        if not os.path.exists(directory):
-            os.makedirs(directory)
-        existing = os.listdir(directory)
-        all = self._generate_clouds()
-        changed = False
-        for new in set(all.keys()) - set(existing):
-            open(f'{directory}/{new}', 'w').write(all[new])
-            changed = True
-        for old in set(existing) - set(all.keys()):
-            old = f'{directory}/{old}'
-            assert os.stat(old).st_nlink == 1, f'{old} has {os.stat(old).st_nlink} links'
-            os.unlink(old)
-            changed = True
-        return changed
+    def network_delete_if_not_used(self, name):
+        if not self.network_exists(name):
+            return
+        ports = self.o.port.list('--device-owner=compute:None',
+                                 '--network', name, '--format=value', '-c', 'ID')
+        if ports.strip() == '':
+            self.o.network.delete(name)
 
-    def region_empty(self):
-        servers = self.o.server.list()
-        images = self.o.image.list('--private')
-        return servers.strip() == '' and images.strip() == ''
+    def subnet_exists(self, name):
+        subnet = self.o.subnet.list('--format=value', '-c', 'Name', '--name', name)
+        return subnet.strip() == name
 
-    def allocate_cloud(self, directory, destination):
-        if os.path.exists(destination):
-            return True
-        for f in sorted(os.listdir(directory)):
-            origin = f'{directory}/{f}'
-            os.link(origin, destination)
-            if (
-                    os.stat(origin).st_nlink == 2 and
-                    OpenStack(self.config_dir, origin).region_empty() and
-                    Heat(self.config_dir, origin).is_working()
-            ):
-                return origin
-            else:
-                os.unlink(destination)
-        return False
+    def subnet_create(self, name, cidr):
+        if not self.subnet_exists(name):
+            self.o.subnet.create('--subnet-range', cidr, '--network', name, name)
+
+    def network_and_subnet_create(self, name, cidr):
+        self.network_create(name)
+        self.subnet_create(name, cidr)
+
+    def server_connected_to_network(self, server, network):
+        port = self.o.port.list('--server', server, '--network', network,
+                                '--format=value', '-c', 'ID')
+        return port.strip() != ''
