@@ -21,7 +21,32 @@ class Service(ABC):
 
         bind = ['bind-host', 'icinga-host', 'postfix-host', 'wazuh-host']
         self.service2hosts['bind'] = bind
-        self.service2hosts['cloud'] = bind[:] + ['cloud-host']
+
+        cloud = bind[:] + ['cloud-host']
+        self.service2hosts['cloud'] = cloud
+
+        forum = bind[:] + ['forum-host']
+        self.service2hosts['forum'] = forum
+
+        weblate = bind[:] + ['weblate-host']
+        self.service2hosts['weblate'] = weblate
+
+        gitlab = bind[:] + ['gitlab-host', 'runner-host']
+        self.service2hosts['gitlab'] = gitlab
+
+        api = gitlab[:] + ['api-host']
+        self.service2hosts['api'] = api
+
+        chat = bind[:] + ['chat-host']
+        self.service2hosts['chat'] = chat
+
+        packages = bind[:] + ['packages-host']
+        self.service2hosts['packages'] = packages
+
+        website = bind[:] + ['website-host']
+        self.service2hosts['website'] = website
+
+        self.service2hosts['pad'] = website
 
     @abstractmethod
     def create_or_update(self):
@@ -75,8 +100,7 @@ class ServiceOpenStack(Service):
         self.maybe_delegate_dns()
         hosts = self.service2hosts[self.args['name']]
         h = openstack.Heat(self.config_dir, self.dotenough.clouds_file)
-        h.create_or_update(hosts, self.dotenough.public_key())
-        h.write_inventory()
+        h.create_missings(hosts, self.dotenough.public_key())
         playbook = ansible_utils.Playbook(self.config_dir, self.share_dir)
         playbook.run([
             f'--private-key={self.dotenough.private_key()}',
